@@ -1,8 +1,8 @@
 /*
  *      filetypes.h - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2005-2010 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
- *      Copyright 2006-2010 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
+ *      Copyright 2005-2012 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2006-2012 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -17,14 +17,14 @@
  *      You should have received a copy of the GNU General Public License
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
- *
- * $Id: filetypes.h 4757 2010-03-12 18:15:48Z ntrel $
  */
 
 
 #ifndef GEANY_FILETYPES_H
 #define GEANY_FILETYPES_H 1
 
+G_BEGIN_DECLS
+
 #include "Scintilla.h"
 #include "ScintillaWidget.h"
 
@@ -33,6 +33,7 @@
 #endif
 
 
+/* Do not change the order, only append. */
 typedef enum
 {
 	GEANY_FILETYPES_NONE = 0,	/* first filetype is always None & must be 0 */
@@ -84,6 +85,11 @@ typedef enum
 	GEANY_FILETYPES_TXT2TAGS,
 	GEANY_FILETYPES_ABC,
 	GEANY_FILETYPES_VERILOG,
+	GEANY_FILETYPES_FORTH,
+	GEANY_FILETYPES_LISP,
+	GEANY_FILETYPES_ERLANG,
+	GEANY_FILETYPES_COBOL,
+	GEANY_FILETYPES_OBJECTIVEC,
 	/* ^ append items here */
 	GEANY_MAX_BUILT_IN_FILETYPES	/* Don't use this, use filetypes_array->len instead */
 }
@@ -96,7 +102,6 @@ typedef enum
 	GEANY_FILETYPE_GROUP_SCRIPT,
 	GEANY_FILETYPE_GROUP_MARKUP,
 	GEANY_FILETYPE_GROUP_MISC,
-	GEANY_FILETYPE_GROUP_CUSTOM,
 	GEANY_FILETYPE_GROUP_COUNT
 }
 GeanyFiletypeGroupID;
@@ -114,8 +119,12 @@ struct GeanyFiletype
 	/** Represents the langType of tagmanager (see the table
 	 * in tagmanager/parsers.h), -1 represents all, -2 none. */
 	langType 		  lang;
-	gchar			 *name;				/**< Short name, e.g. "C". */
-	gchar			 *title;			/**< Shown in the file open dialog. E.g. "C source file". */
+	/** Untranslated short name, such as "C", "None".
+	 * Must not be translated as it's used for hash table lookups - use
+	 * filetypes_get_display_name() instead. */
+	gchar			 *name;
+	/** Shown in the file open dialog, such as "C source file". */
+	gchar			 *title;
 	gchar			 *extension;		/**< Default file extension for new files, or @c NULL. */
 	gchar			**pattern;			/**< Array of filename-matching wildcard strings. */
 	gchar			 *context_action_cmd;
@@ -125,12 +134,19 @@ struct GeanyFiletype
 	GeanyFiletypeGroupID group;
 	gchar			 *error_regex_string;
 	GeanyFiletype	 *lexer_filetype;
+	gchar			 *mime_type;
+	GdkPixbuf		 *icon;
+	gchar			 *comment_single; /* single-line comment */
+	/* filetype indent settings, -1 if not set */
+	gint			  indent_type;
+	gint			  indent_width;
 
 	struct GeanyFiletypePrivate	*priv;	/* must be last, append fields before this item */
 #ifdef GEANY_PRIVATE
 	/* Do not use following fields in plugins */
-	GeanyBuildCommand *filecmds;	/* these need to be visible since used in build.c so not in private part */
-	GeanyBuildCommand *ftdefcmds;	/* filetype dependent defaults for non_ft commands */
+	/* TODO: move these fields into filetypesprivate.h */
+	GeanyBuildCommand *filecmds;
+	GeanyBuildCommand *ftdefcmds;
 	GeanyBuildCommand *execcmds;
 	GeanyBuildCommand *homefilecmds;
 	GeanyBuildCommand *homeexeccmds;
@@ -139,7 +155,6 @@ struct GeanyFiletype
 	gint			 project_list_entry;
 	gchar			 *projerror_regex_string;
 	gchar			 *homeerror_regex_string;
-	gboolean		  home_save_needed;
 #endif
 };
 
@@ -162,13 +177,17 @@ void filetypes_init(void);
 
 void filetypes_init_types(void);
 
-void filetypes_read_extensions(void);
+void filetypes_reload_extensions(void);
 
 void filetypes_reload(void);
 
 
 GeanyFiletype *filetypes_index(gint idx);
 
+const GSList *filetypes_get_sorted_by_name(void);
+
+const gchar *filetypes_get_display_name(GeanyFiletype *ft);
+
 GeanyFiletype *filetypes_detect_from_document(GeanyDocument *doc);
 
 GeanyFiletype *filetypes_detect_from_extension(const gchar *utf8_filename);
@@ -177,9 +196,9 @@ GeanyFiletype *filetypes_detect_from_file(const gchar *utf8_filename);
 
 void filetypes_free_types(void);
 
-void filetypes_load_config(gint ft_id, gboolean reload);
+void filetypes_load_config(guint ft_id, gboolean reload);
 
-void filetypes_save_commands(void);
+void filetypes_save_commands(GeanyFiletype *ft);
 
 void filetypes_select_radio_item(const GeanyFiletype *ft);
 
@@ -187,11 +206,14 @@ GtkFileFilter *filetypes_create_file_filter(const GeanyFiletype *ft);
 
 GtkFileFilter *filetypes_create_file_filter_all_source(void);
 
-gchar *filetypes_get_conf_extension(gint filetype_idx);
-
 gboolean filetype_has_tags(GeanyFiletype *ft);
 
 gboolean filetypes_parse_error_message(GeanyFiletype *ft, const gchar *message,
 		gchar **filename, gint *line);
 
+gboolean filetype_get_comment_open_close(const GeanyFiletype *ft, gboolean single_first,
+		const gchar **co, const gchar **cc);
+
+G_END_DECLS
+
 #endif
