@@ -1,8 +1,8 @@
 /*
  *      keyfile.c - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2005-2010 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
- *      Copyright 2006-2010 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
+ *      Copyright 2005-2012 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2006-2012 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -17,8 +17,6 @@
  *      You should have received a copy of the GNU General Public License
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
- *
- * $Id: keyfile.c 4946 2010-05-22 19:40:12Z eht16 $
  */
 
 /*
@@ -83,6 +81,8 @@
 #define GEANY_DEFAULT_FONT_EDITOR		"Monospace 10"
 #define GEANY_TOGGLE_MARK				"~ "
 #define GEANY_MAX_AUTOCOMPLETE_WORDS	30
+#define GEANY_MAX_SYMBOLS_UPDATE_FREQ	250
+#define GEANY_DEFAULT_FILETYPE_REGEX    "-\\*-\\s*([^\\s]+)\\s*-\\*-"
 
 
 static gchar *scribble_text = NULL;
@@ -91,10 +91,20 @@ static GPtrArray *session_files = NULL;
 static gint session_notebook_page;
 static gint hpan_position;
 static gint vpan_position;
+static const gchar atomic_file_saving_key[] = "use_atomic_file_saving";
 
 static GPtrArray *keyfile_groups = NULL;
+
 GPtrArray *pref_groups = NULL;
 
+static struct
+{
+	gint number_ft_menu_items;
+	gint number_non_ft_menu_items;
+	gint number_exec_menu_items;
+}
+build_menu_prefs;
+
 
 /* The group will be free'd on quitting.
  * @param for_prefs_dialog is whether the group also has Prefs dialog items. */
@@ -107,6 +117,14 @@ void configuration_add_pref_group(struct StashGroup *group, gboolean for_prefs_d
 }
 
 
+/* The group will be free'd on quitting. */
+void configuration_add_various_pref_group(struct StashGroup *group)
+{
+	configuration_add_pref_group(group, TRUE);
+	stash_group_set_various(group, TRUE);
+}
+
+
 static void init_pref_groups(void)
 {
 	StashGroup *group;
@@ -121,6 +139,8 @@ static void init_pref_groups(void)
 
 	stash_group_add_toggle_button(group, &interface_prefs.notebook_double_click_hides_widgets,
 		"notebook_double_click_hides_widgets", FALSE, "check_double_click_hides_widgets");
+	stash_group_add_toggle_button(group, &file_prefs.tab_close_switch_to_mru,
+		"tab_close_switch_to_mru", FALSE, "check_tab_close_switch_to_mru");
 	stash_group_add_integer(group, &interface_prefs.tab_pos_sidebar, "tab_pos_sidebar", GTK_POS_TOP);
 	stash_group_add_radio_buttons(group, &interface_prefs.sidebar_pos,
 		"sidebar_pos", GTK_POS_LEFT,
@@ -132,14 +152,14 @@ static void init_pref_groups(void)
 	stash_group_add_toggle_button(group, &interface_prefs.highlighting_invert_all,
 		"highlighting_invert_all", FALSE, "check_highlighting_invert");
 
-	stash_group_add_toggle_button(group, &search_prefs.suppress_dialogs,
-		"pref_main_suppress_search_dialogs", FALSE, "check_ask_suppress_search_dialogs");
 	stash_group_add_toggle_button(group, &search_prefs.use_current_word,
 		"pref_main_search_use_current_word", TRUE, "check_search_use_current_word");
 
 	/* editor */
 	stash_group_add_toggle_button(group, &editor_prefs.indentation->detect_type,
-		"check_detect_indent", FALSE, "check_detect_indent");
+		"check_detect_indent", FALSE, "check_detect_indent_type");
+	stash_group_add_toggle_button(group, &editor_prefs.indentation->detect_width,
+		"detect_indent_width", FALSE, "check_detect_indent_width");
 	stash_group_add_toggle_button(group, &editor_prefs.use_tab_to_indent,
 		"use_tab_to_indent", TRUE, "check_tab_key_indents");
 	stash_group_add_spin_button_integer(group, &editor_prefs.indentation->width,
@@ -165,6 +185,8 @@ static void init_pref_groups(void)
 	stash_group_add_spin_button_integer(group, (gint*)&editor_prefs.autocompletion_max_entries,
 		"autocompletion_max_entries", GEANY_MAX_AUTOCOMPLETE_WORDS,
 		"spin_autocompletion_max_entries");
+	stash_group_add_spin_button_integer(group, (gint*)&editor_prefs.autocompletion_update_freq,
+		"autocompletion_update_freq", GEANY_MAX_SYMBOLS_UPDATE_FREQ, "spin_symbol_update_freq");
 	stash_group_add_string(group, &editor_prefs.color_scheme,
 		"color_scheme", NULL);
 
@@ -174,10 +196,9 @@ static void init_pref_groups(void)
 	stash_group_add_spin_button_integer(group, &file_prefs.disk_check_timeout,
 		"disk_check_timeout", GEANY_DISK_CHECK_TIMEOUT, "spin_disk_check");
 
-	/* hidden prefs (don't overwrite them so users can edit them manually) */
+	/* various geany prefs */
 	group = stash_group_new(PACKAGE);
-	configuration_add_pref_group(group, FALSE);
-	stash_group_set_write_once(group, TRUE);
+	configuration_add_various_pref_group(group);
 
 	stash_group_add_boolean(group, &editor_prefs.show_scrollbars,
 		"show_editor_scrollbars", TRUE);
@@ -187,17 +208,32 @@ static void init_pref_groups(void)
 		"use_gtk_word_boundaries", TRUE);
 	stash_group_add_boolean(group, &editor_prefs.complete_snippets_whilst_editing,
 		"complete_snippets_whilst_editing", FALSE);
-	stash_group_add_boolean(group, &interface_prefs.show_symbol_list_expanders,
-		"show_symbol_list_expanders", TRUE);
-	stash_group_add_boolean(group, &interface_prefs.compiler_tab_autoscroll,
-		"compiler_tab_autoscroll", TRUE);
-	stash_group_add_boolean(group, &ui_prefs.allow_always_save,
-		"allow_always_save", FALSE);
 	stash_group_add_boolean(group, &file_prefs.use_safe_file_saving,
-		"use_safe_file_saving", FALSE);
+		atomic_file_saving_key, FALSE);
+	stash_group_add_boolean(group, &file_prefs.gio_unsafe_save_backup,
+		"gio_unsafe_save_backup", FALSE);
+	stash_group_add_boolean(group, &file_prefs.use_gio_unsafe_file_saving,
+		"use_gio_unsafe_file_saving", TRUE);
 	/* for backwards-compatibility */
 	stash_group_add_integer(group, &editor_prefs.indentation->hard_tab_width,
 		"indent_hard_tab_width", 8);
+	stash_group_add_integer(group, (gint*)&search_prefs.find_selection_type,
+		"find_selection_type", GEANY_FIND_SEL_CURRENT_WORD);
+	stash_group_add_string(group, &file_prefs.extract_filetype_regex,
+		"extract_filetype_regex", GEANY_DEFAULT_FILETYPE_REGEX);
+
+	/* Note: Interface-related various prefs are in ui_init_prefs() */
+
+	/* various build-menu prefs */
+	group = stash_group_new("build-menu");
+	configuration_add_various_pref_group(group);
+
+	stash_group_add_integer(group, &build_menu_prefs.number_ft_menu_items,
+		"number_ft_menu_items", 0);
+	stash_group_add_integer(group, &build_menu_prefs.number_non_ft_menu_items,
+		"number_non_ft_menu_items", 0);
+	stash_group_add_integer(group, &build_menu_prefs.number_exec_menu_items,
+		"number_exec_menu_items", 0);
 }
 
 
@@ -257,20 +293,16 @@ static gchar *get_session_file_string(GeanyDocument *doc)
 {
 	gchar *fname;
 	gchar *locale_filename;
+	gchar *escaped_filename;
 	GeanyFiletype *ft = doc->file_type;
 
 	if (ft == NULL) /* can happen when saving a new file when quitting */
 		ft = filetypes[GEANY_FILETYPES_NONE];
 
 	locale_filename = utils_get_locale_from_utf8(doc->file_name);
-	/* If the filename contains any ';' (semi-colons) we need to escape them otherwise
-	 * g_key_file_get_string_list() would fail reading them, so we replace them before
-	 * writing with usual colons which must never appear in a filename and replace them
-	 * back when we read the file again from the file.
-	 * (g_path_skip_root() to skip C:\... on Windows) */
-	g_strdelimit((gchar*) utils_path_skip_root(locale_filename), ";", ':');
+	escaped_filename = g_uri_escape_string(locale_filename, NULL, TRUE);
 
-	fname = g_strdup_printf("%d;%s;%d;%d;%d;%d;%d;%s;%d",
+	fname = g_strdup_printf("%d;%s;%d;%d;%d;%d;%d;%s;%d;%d",
 		sci_get_current_position(doc->editor->sci),
 		ft->name,
 		doc->readonly,
@@ -278,8 +310,10 @@ static gchar *get_session_file_string(GeanyDocument *doc)
 		doc->editor->indent_type,
 		doc->editor->auto_indent,
 		doc->editor->line_wrapping,
-		locale_filename,
-		doc->editor->line_breaking);
+		escaped_filename,
+		doc->editor->line_breaking,
+		doc->editor->indent_width);
+	g_free(escaped_filename);
 	g_free(locale_filename);
 	return fname;
 }
@@ -296,7 +330,7 @@ void configuration_save_session_files(GKeyFile *config)
 	npage = gtk_notebook_get_current_page(GTK_NOTEBOOK(main_widgets.notebook));
 	g_key_file_set_integer(config, "files", "current_page", npage);
 
-	/* store the filenames to reopen them the next time */
+	/* store the filenames in the notebook tab order to reopen them the next time */
 	max = gtk_notebook_get_n_pages(GTK_NOTEBOOK(main_widgets.notebook));
 	for (i = 0; i < max; i++)
 	{
@@ -329,6 +363,14 @@ void configuration_save_session_files(GKeyFile *config)
 			i++;
 		}
 	}
+
+#ifdef HAVE_VTE
+	if (vte_info.have_vte)
+	{
+		vte_get_working_directory();	/* refresh vte_info.dir */
+		g_key_file_set_string(config, "VTE", "last_dir", vte_info.dir);
+	}
+#endif
 }
 
 
@@ -362,10 +404,6 @@ static void save_dialog_prefs(GKeyFile *config)
 	g_key_file_set_boolean(config, PACKAGE, "tab_order_beside", file_prefs.tab_order_beside);
 	g_key_file_set_integer(config, PACKAGE, "tab_pos_editor", interface_prefs.tab_pos_editor);
 	g_key_file_set_integer(config, PACKAGE, "tab_pos_msgwin", interface_prefs.tab_pos_msgwin);
-	g_key_file_set_boolean(config, PACKAGE, "msgwin_status_visible", interface_prefs.msgwin_status_visible);
-	g_key_file_set_boolean(config, PACKAGE, "msgwin_compiler_visible", interface_prefs.msgwin_compiler_visible);
-	g_key_file_set_boolean(config, PACKAGE, "msgwin_messages_visible", interface_prefs.msgwin_messages_visible);
-	g_key_file_set_boolean(config, PACKAGE, "msgwin_scribble_visible", interface_prefs.msgwin_scribble_visible);
 	g_key_file_set_boolean(config, PACKAGE, "use_native_windows_dialogs", interface_prefs.use_native_windows_dialogs);
 
 	/* display */
@@ -374,9 +412,9 @@ static void save_dialog_prefs(GKeyFile *config)
 	g_key_file_set_boolean(config, PACKAGE, "show_line_endings", editor_prefs.show_line_endings);
 	g_key_file_set_boolean(config, PACKAGE, "show_markers_margin", editor_prefs.show_markers_margin);
 	g_key_file_set_boolean(config, PACKAGE, "show_linenumber_margin", editor_prefs.show_linenumber_margin);
-	g_key_file_set_boolean(config, PACKAGE, "long_line_enabled", editor_prefs.long_line_global_enabled);
-	g_key_file_set_integer(config, PACKAGE, "long_line_type", editor_prefs.long_line_global_type);
-	g_key_file_set_integer(config, PACKAGE, "long_line_column", editor_prefs.long_line_global_column);
+	g_key_file_set_boolean(config, PACKAGE, "long_line_enabled", editor_prefs.long_line_enabled);
+	g_key_file_set_integer(config, PACKAGE, "long_line_type", editor_prefs.long_line_type);
+	g_key_file_set_integer(config, PACKAGE, "long_line_column", editor_prefs.long_line_column);
 	g_key_file_set_string(config, PACKAGE, "long_line_color", editor_prefs.long_line_color);
 
 	/* editor */
@@ -406,6 +444,7 @@ static void save_dialog_prefs(GKeyFile *config)
 		g_key_file_set_string(config, PACKAGE, "pref_editor_default_open_encoding", encodings[file_prefs.default_open_encoding].charset);
 	g_key_file_set_integer(config, PACKAGE, "default_eol_character", file_prefs.default_eol_character);
 	g_key_file_set_boolean(config, PACKAGE, "pref_editor_new_line", file_prefs.final_new_line);
+	g_key_file_set_boolean(config, PACKAGE, "pref_editor_ensure_convert_line_endings", file_prefs.ensure_convert_new_lines);
 	g_key_file_set_boolean(config, PACKAGE, "pref_editor_replace_tabs", file_prefs.replace_tabs);
 	g_key_file_set_boolean(config, PACKAGE, "pref_editor_trail_space", file_prefs.strip_trailing_spaces);
 
@@ -448,7 +487,7 @@ static void save_dialog_prefs(GKeyFile *config)
 	/* VTE */
 #ifdef HAVE_VTE
 	g_key_file_set_boolean(config, "VTE", "load_vte", vte_info.load_vte);
-	if (vte_info.load_vte && vc != NULL)
+	if (vte_info.have_vte)
 	{
 		gchar *tmp_string;
 
@@ -457,6 +496,8 @@ static void save_dialog_prefs(GKeyFile *config)
 		if (!g_key_file_has_key(config, "VTE", "send_selection_unsafe", NULL))	/* hidden */
 			g_key_file_set_boolean(config, "VTE", "send_selection_unsafe",
 				vc->send_selection_unsafe);
+		if (!g_key_file_has_key(config, "VTE", "send_cmd_prefix", NULL))	/* hidden */
+			g_key_file_set_string(config, "VTE", "send_cmd_prefix", vc->send_cmd_prefix);
 		g_key_file_set_string(config, "VTE", "font", vc->font);
 		g_key_file_set_boolean(config, "VTE", "scroll_on_key", vc->scroll_on_key);
 		g_key_file_set_boolean(config, "VTE", "scroll_on_out", vc->scroll_on_out);
@@ -475,8 +516,6 @@ static void save_dialog_prefs(GKeyFile *config)
 		tmp_string = utils_get_hex_from_color(vc->colour_back);
 		g_key_file_set_string(config, "VTE", "colour_back", tmp_string);
 		g_free(tmp_string);
-		vte_get_working_directory();	/* refresh vte_info.dir */
-		g_key_file_set_string(config, "VTE", "last_dir", vte_info.dir);
 	}
 #endif
 }
@@ -509,6 +548,8 @@ static void save_ui_prefs(GKeyFile *config)
 
 	if (prefs.save_winpos)
 	{
+		GdkWindowState wstate;
+
 		g_key_file_set_integer(config, PACKAGE, "treeview_position",
 				gtk_paned_get_position(GTK_PANED(ui_lookup_widget(main_widgets.window, "hpaned1"))));
 		g_key_file_set_integer(config, PACKAGE, "msgwindow_position",
@@ -516,11 +557,8 @@ static void save_ui_prefs(GKeyFile *config)
 
 		gtk_window_get_position(GTK_WINDOW(main_widgets.window), &ui_prefs.geometry[0], &ui_prefs.geometry[1]);
 		gtk_window_get_size(GTK_WINDOW(main_widgets.window), &ui_prefs.geometry[2], &ui_prefs.geometry[3]);
-		if (gdk_window_get_state(main_widgets.window->window) & GDK_WINDOW_STATE_MAXIMIZED)
-			ui_prefs.geometry[4] = 1;
-		else
-			ui_prefs.geometry[4] = 0;
-
+		wstate = gdk_window_get_state(gtk_widget_get_window(main_widgets.window));
+		ui_prefs.geometry[4] = (wstate & GDK_WINDOW_STATE_MAXIMIZED) ? 1 : 0;
 		g_key_file_set_integer_list(config, PACKAGE, "geometry", ui_prefs.geometry, 5);
 	}
 
@@ -529,6 +567,8 @@ static void save_ui_prefs(GKeyFile *config)
 	{
 		g_key_file_set_string_list(config, PACKAGE, "custom_commands",
 				(const gchar**) ui_prefs.custom_commands, g_strv_length(ui_prefs.custom_commands));
+		g_key_file_set_string_list(config, PACKAGE, "custom_commands_labels",
+				(const gchar**) ui_prefs.custom_commands_labels, g_strv_length(ui_prefs.custom_commands_labels));
 	}
 }
 
@@ -552,6 +592,13 @@ void configuration_save(void)
 
 	if (cl_options.load_session)
 		configuration_save_session_files(config);
+#ifdef HAVE_VTE
+	else if (vte_info.have_vte)
+	{
+		vte_get_working_directory();	/* refresh vte_info.dir */
+		g_key_file_set_string(config, "VTE", "last_dir", vte_info.dir);
+	}
+#endif
 
 	/* write the file */
 	data = g_key_file_to_data(config, NULL, NULL);
@@ -621,15 +668,44 @@ void configuration_load_session_files(GKeyFile *config, gboolean read_recent_fil
 		g_ptr_array_add(session_files, tmp_array);
 		i++;
 	}
+
+#ifdef HAVE_VTE
+	/* BUG: after loading project at startup, closing project doesn't restore old VTE path */
+	if (vte_info.have_vte)
+	{
+		gchar *tmp_string = utils_get_setting_string(config, "VTE", "last_dir", NULL);
+		vte_cwd(tmp_string,TRUE);
+		g_free(tmp_string);
+	}
+#endif
 }
 
 
+/* note: new settings should be added in init_pref_groups() */
 static void load_dialog_prefs(GKeyFile *config)
 {
 	gchar *tmp_string, *tmp_string2;
 	const gchar *default_charset = NULL;
 
-	/* new settings should be added in init_pref_groups() */
+	/* compatibility with Geany 0.20 */
+	if (!g_key_file_has_key(config, PACKAGE, atomic_file_saving_key, NULL))
+	{
+		g_key_file_set_boolean(config, PACKAGE, atomic_file_saving_key,
+			utils_get_setting_boolean(config, PACKAGE, "use_safe_file_saving", FALSE));
+	}
+
+	/* compatibility with Geany 0.21 */
+	{
+		gboolean suppress_search_dialogs = utils_get_setting_boolean(config, PACKAGE, "pref_main_suppress_search_dialogs", FALSE);
+
+		if (!g_key_file_has_key(config, "search", "pref_search_always_wrap", NULL))
+			g_key_file_set_boolean(config, "search", "pref_search_always_wrap", suppress_search_dialogs);
+
+		if (!g_key_file_has_key(config, "search", "pref_search_hide_find_dialog", NULL))
+			g_key_file_set_boolean(config, "search", "pref_search_hide_find_dialog", suppress_search_dialogs);
+	}
+
+	/* read stash prefs */
 	settings_action(config, SETTING_READ);
 
 	/* general */
@@ -656,22 +732,18 @@ static void load_dialog_prefs(GKeyFile *config)
 	interface_prefs.editor_font = utils_get_setting_string(config, PACKAGE, "editor_font", GEANY_DEFAULT_FONT_EDITOR);
 	interface_prefs.tagbar_font = utils_get_setting_string(config, PACKAGE, "tagbar_font", GEANY_DEFAULT_FONT_SYMBOL_LIST);
 	interface_prefs.msgwin_font = utils_get_setting_string(config, PACKAGE, "msgwin_font", GEANY_DEFAULT_FONT_MSG_WINDOW);
-	interface_prefs.msgwin_status_visible = utils_get_setting_boolean(config, PACKAGE, "msgwin_status_visible", TRUE);
-	interface_prefs.msgwin_compiler_visible = utils_get_setting_boolean(config, PACKAGE, "msgwin_compiler_visible", TRUE);
-	interface_prefs.msgwin_messages_visible = utils_get_setting_boolean(config, PACKAGE, "msgwin_messages_visible", TRUE);
-	interface_prefs.msgwin_scribble_visible = utils_get_setting_boolean(config, PACKAGE, "msgwin_scribble_visible", TRUE);
 	interface_prefs.use_native_windows_dialogs = utils_get_setting_boolean(config, PACKAGE, "use_native_windows_dialogs", FALSE);
 
 	/* display, editor */
-	editor_prefs.long_line_global_enabled = utils_get_setting_boolean(config, PACKAGE, "long_line_enabled", TRUE);
-	editor_prefs.long_line_global_type = utils_get_setting_integer(config, PACKAGE, "long_line_type", 0);
-	if (editor_prefs.long_line_global_type == 2) /* backward compatibility */
+	editor_prefs.long_line_enabled = utils_get_setting_boolean(config, PACKAGE, "long_line_enabled", TRUE);
+	editor_prefs.long_line_type = utils_get_setting_integer(config, PACKAGE, "long_line_type", 0);
+	if (editor_prefs.long_line_type == 2) /* backward compatibility */
 	{
-		editor_prefs.long_line_global_type = 0;
-		editor_prefs.long_line_global_enabled = FALSE;
+		editor_prefs.long_line_type = 0;
+		editor_prefs.long_line_enabled = FALSE;
 	}
 	editor_prefs.long_line_color = utils_get_setting_string(config, PACKAGE, "long_line_color", "#C2EBC2");
-	editor_prefs.long_line_global_column = utils_get_setting_integer(config, PACKAGE, "long_line_column", 72);
+	editor_prefs.long_line_column = utils_get_setting_integer(config, PACKAGE, "long_line_column", 72);
 	editor_prefs.symbolcompletion_min_chars = utils_get_setting_integer(config, PACKAGE, "symbolcompletion_min_chars", GEANY_MIN_SYMBOLLIST_CHARS);
 	editor_prefs.symbolcompletion_max_height = utils_get_setting_integer(config, PACKAGE, "symbolcompletion_max_height", GEANY_MAX_SYMBOLLIST_HEIGHT);
 	editor_prefs.line_wrapping = utils_get_setting_boolean(config, PACKAGE, "line_wrapping", FALSE); /* default is off for better performance */
@@ -724,6 +796,7 @@ static void load_dialog_prefs(GKeyFile *config)
 	}
 	file_prefs.default_eol_character = utils_get_setting_integer(config, PACKAGE, "default_eol_character", GEANY_DEFAULT_EOL_CHARACTER);
 	file_prefs.replace_tabs = utils_get_setting_boolean(config, PACKAGE, "pref_editor_replace_tabs", FALSE);
+	file_prefs.ensure_convert_new_lines = utils_get_setting_boolean(config, PACKAGE, "pref_editor_ensure_convert_line_endings", FALSE);
 	file_prefs.final_new_line = utils_get_setting_boolean(config, PACKAGE, "pref_editor_new_line", TRUE);
 	file_prefs.strip_trailing_spaces = utils_get_setting_boolean(config, PACKAGE, "pref_editor_trail_space", FALSE);
 
@@ -764,9 +837,10 @@ static void load_dialog_prefs(GKeyFile *config)
 		vc->font = utils_get_setting_string(config, "VTE", "font", "Monospace 10");
 		vc->scroll_on_key = utils_get_setting_boolean(config, "VTE", "scroll_on_key", TRUE);
 		vc->scroll_on_out = utils_get_setting_boolean(config, "VTE", "scroll_on_out", TRUE);
-		vc->enable_bash_keys = utils_get_setting_boolean(config, "VTE", "enable_bash_keys", FALSE);
+		vc->enable_bash_keys = utils_get_setting_boolean(config, "VTE", "enable_bash_keys", TRUE);
 		vc->ignore_menu_bar_accel = utils_get_setting_boolean(config, "VTE", "ignore_menu_bar_accel", FALSE);
 		vc->follow_path = utils_get_setting_boolean(config, "VTE", "follow_path", FALSE);
+		vc->send_cmd_prefix = utils_get_setting_string(config, "VTE", "send_cmd_prefix", "");
 		vc->run_in_vte = utils_get_setting_boolean(config, "VTE", "run_in_vte", FALSE);
 		vc->skip_run_script = utils_get_setting_boolean(config, "VTE", "skip_run_script", FALSE);
 		vc->cursor_blinks = utils_get_setting_boolean(config, "VTE", "cursor_blinks", FALSE);
@@ -807,9 +881,9 @@ static void load_dialog_prefs(GKeyFile *config)
 	tool_prefs.context_action_cmd = utils_get_setting_string(config, PACKAGE, "context_action_cmd", "");
 
 	/* build menu */
-	build_set_group_count(GEANY_GBG_FT, utils_get_setting_integer(config, "build-menu", "number_ft_menu_items", 0));
-	build_set_group_count(GEANY_GBG_NON_FT, utils_get_setting_integer(config, "build-menu", "number_non_ft_menu_items", 0));
-	build_set_group_count(GEANY_GBG_EXEC, utils_get_setting_integer(config, "build-menu", "number_exec_menu_items", 0));
+	build_set_group_count(GEANY_GBG_FT, build_menu_prefs.number_ft_menu_items);
+	build_set_group_count(GEANY_GBG_NON_FT, build_menu_prefs.number_non_ft_menu_items);
+	build_set_group_count(GEANY_GBG_EXEC, build_menu_prefs.number_exec_menu_items);
 	build_load_menu(config, GEANY_BCS_PREF, NULL);
 
 	/* printing */
@@ -824,11 +898,7 @@ static void load_dialog_prefs(GKeyFile *config)
 	g_free(tmp_string);
 	g_free(tmp_string2);
 
-#if GTK_CHECK_VERSION(2, 10, 0)
 	printing_prefs.use_gtk_printing = utils_get_setting_boolean(config, "printing", "use_gtk_printing", TRUE);
-#else
-	printing_prefs.use_gtk_printing = utils_get_setting_boolean(config, "printing", "use_gtk_printing", FALSE);
-#endif
 	printing_prefs.print_line_numbers = utils_get_setting_boolean(config, "printing", "print_line_numbers", TRUE);
 	printing_prefs.print_page_numbers = utils_get_setting_boolean(config, "printing", "print_page_numbers", TRUE);
 	printing_prefs.print_page_header = utils_get_setting_boolean(config, "printing", "print_page_header", TRUE);
@@ -840,45 +910,65 @@ static void load_dialog_prefs(GKeyFile *config)
 static void load_ui_prefs(GKeyFile *config)
 {
 	gint *geo;
-	GError *error = NULL;
+	gsize geo_len;
 
 	ui_prefs.sidebar_visible = utils_get_setting_boolean(config, PACKAGE, "sidebar_visible", TRUE);
 	ui_prefs.msgwindow_visible = utils_get_setting_boolean(config, PACKAGE, "msgwindow_visible", TRUE);
 	ui_prefs.fullscreen = utils_get_setting_boolean(config, PACKAGE, "fullscreen", FALSE);
 	ui_prefs.custom_date_format = utils_get_setting_string(config, PACKAGE, "custom_date_format", "");
 	ui_prefs.custom_commands = g_key_file_get_string_list(config, PACKAGE, "custom_commands", NULL, NULL);
+	ui_prefs.custom_commands_labels = g_key_file_get_string_list(config, PACKAGE, "custom_commands_labels", NULL, NULL);
+
+	/* sanitize custom commands labels */
+	if (ui_prefs.custom_commands || ui_prefs.custom_commands_labels)
+	{
+		guint i;
+		guint cc_len = ui_prefs.custom_commands ? g_strv_length(ui_prefs.custom_commands) : 0;
+		guint cc_labels_len = ui_prefs.custom_commands_labels ? g_strv_length(ui_prefs.custom_commands_labels) : 0;
+
+		/* not enough items, resize and fill */
+		if (cc_labels_len < cc_len)
+		{
+			ui_prefs.custom_commands_labels = g_realloc(ui_prefs.custom_commands_labels,
+					(cc_len + 1) * sizeof *ui_prefs.custom_commands_labels);
+			for (i = cc_labels_len; i < cc_len; i++)
+				ui_prefs.custom_commands_labels[i] = g_strdup("");
+			ui_prefs.custom_commands_labels[cc_len] = NULL;
+		}
+		/* too many items, cut off */
+		else if (cc_labels_len > cc_len)
+		{
+			for (i = cc_len; i < cc_labels_len; i++)
+			{
+				g_free(ui_prefs.custom_commands_labels[i]);
+				ui_prefs.custom_commands_labels[i] = NULL;
+			}
+		}
+	}
 
 	scribble_text = utils_get_setting_string(config, PACKAGE, "scribble_text",
 				_("Type here what you want, use it as a notice/scratch board"));
 	scribble_pos = utils_get_setting_integer(config, PACKAGE, "scribble_pos", -1);
 
-	geo = g_key_file_get_integer_list(config, PACKAGE, "geometry", NULL, &error);
-	if (error)
+	geo = g_key_file_get_integer_list(config, PACKAGE, "geometry", &geo_len, NULL);
+	if (! geo || geo_len < 5)
 	{
 		ui_prefs.geometry[0] = -1;
-		g_error_free(error);
-		error = NULL;
+		ui_prefs.geometry[1] = -1;
+		ui_prefs.geometry[2] = -1;
+		ui_prefs.geometry[3] = -1;
+		ui_prefs.geometry[4] = 0;
 	}
 	else
 	{
-		gint i;
-
-		ui_prefs.geometry[0] = geo[0];
-		ui_prefs.geometry[1] = geo[1];
-		ui_prefs.geometry[2] = geo[2];
-		ui_prefs.geometry[3] = geo[3];
-		ui_prefs.geometry[4] = geo[4];
-
 		/* don't use insane values but when main windows was maximized last time, pos might be
 		 * negative (due to differences in root window and window decorations) */
-		if (ui_prefs.geometry[4] != 1)
-		{
-			for (i = 2; i < 4; i++)
-			{
-				if (ui_prefs.geometry[i] < -1)
-					ui_prefs.geometry[i] = -1;
-			}
-		}
+		/* quitting when minimized can make pos -32000, -32000 on Windows! */
+		ui_prefs.geometry[0] = MAX(-1, geo[0]);
+		ui_prefs.geometry[1] = MAX(-1, geo[1]);
+		ui_prefs.geometry[2] = MAX(-1, geo[2]);
+		ui_prefs.geometry[3] = MAX(-1, geo[3]);
+		ui_prefs.geometry[4] = geo[4] != 0;
 	}
 	hpan_position = utils_get_setting_integer(config, PACKAGE, "treeview_position", 156);
 	vpan_position = utils_get_setting_integer(config, PACKAGE, "msgwindow_position", (geo) ?
@@ -918,31 +1008,31 @@ void configuration_save_default_session(void)
  */
 void configuration_reload_default_session(void)
 {
-	gchar *configfile = g_strconcat(app->configdir, G_DIR_SEPARATOR_S, "geany.conf", NULL);
+	gchar *configfile = g_build_filename(app->configdir, "geany.conf", NULL);
 	GKeyFile *config = g_key_file_new();
 
 	g_key_file_load_from_file(config, configfile, G_KEY_FILE_NONE, NULL);
+	g_free(configfile);
 
 	configuration_load_session_files(config, FALSE);
 
 	g_key_file_free(config);
-	g_free(configfile);
 }
 
 
 gboolean configuration_load(void)
 {
-	gchar *configfile = g_strconcat(app->configdir, G_DIR_SEPARATOR_S, "geany.conf", NULL);
+	gchar *configfile = g_build_filename(app->configdir, "geany.conf", NULL);
 	GKeyFile *config = g_key_file_new();
 
-	if (! g_file_test(configfile, G_FILE_TEST_IS_REGULAR | G_FILE_TEST_IS_SYMLINK))
+	if (! g_file_test(configfile, G_FILE_TEST_IS_REGULAR))
 	{	/* config file does not (yet) exist, so try to load a global config file which may be */
 		/* created by distributors */
-		geany_debug("No config file found, try to use global configuration.");
-		setptr(configfile, g_strconcat(app->datadir, G_DIR_SEPARATOR_S "geany.conf", NULL));
+		geany_debug("No user config file found, trying to use global configuration.");
+		SETPTR(configfile, g_build_filename(app->datadir, "geany.conf", NULL));
 	}
-
 	g_key_file_load_from_file(config, configfile, G_KEY_FILE_NONE, NULL);
+	g_free(configfile);
 
 	load_dialog_prefs(config);
 	load_ui_prefs(config);
@@ -953,7 +1043,6 @@ gboolean configuration_load(void)
 	g_signal_emit_by_name(geany_object, "load-settings", config);
 
 	g_key_file_free(config);
-	g_free(configfile);
 	return TRUE;
 }
 
@@ -963,6 +1052,7 @@ static gboolean open_session_file(gchar **tmp, guint len)
 	guint pos;
 	const gchar *ft_name;
 	gchar *locale_filename;
+	gchar *unescaped_filename;
 	gint enc_idx, indent_type;
 	gboolean ro, auto_indent, line_wrapping;
 	/** TODO when we have a global pref for line breaking, use its value */
@@ -977,14 +1067,13 @@ static gboolean open_session_file(gchar **tmp, guint len)
 	auto_indent = atoi(tmp[5]);
 	line_wrapping = atoi(tmp[6]);
 	/* try to get the locale equivalent for the filename */
-	locale_filename = utils_get_locale_from_utf8(tmp[7]);
-	/* replace ':' back with ';' (see get_session_file_string for details) */
-	g_strdelimit((gchar*) utils_path_skip_root(locale_filename), ":", ';');
+	unescaped_filename = g_uri_unescape_string(tmp[7], NULL);
+	locale_filename = utils_get_locale_from_utf8(unescaped_filename);
 
 	if (len > 8)
 		line_breaking = atoi(tmp[8]);
 
-	if (g_file_test(locale_filename, G_FILE_TEST_IS_REGULAR | G_FILE_TEST_IS_SYMLINK))
+	if (g_file_test(locale_filename, G_FILE_TEST_IS_REGULAR))
 	{
 		GeanyFiletype *ft = filetypes_lookup_by_name(ft_name);
 		GeanyDocument *doc = document_open_file_full(
@@ -994,7 +1083,11 @@ static gboolean open_session_file(gchar **tmp, guint len)
 
 		if (doc)
 		{
-			editor_set_indent_type(doc->editor, indent_type);
+			gint indent_width = doc->editor->indent_width;
+
+			if (len > 9)
+				indent_width = atoi(tmp[9]);
+			editor_set_indent(doc->editor, indent_type, indent_width);
 			editor_set_line_wrapping(doc->editor, line_wrapping);
 			doc->editor->line_breaking = line_breaking;
 			doc->editor->auto_indent = auto_indent;
@@ -1007,6 +1100,7 @@ static gboolean open_session_file(gchar **tmp, guint len)
 	}
 
 	g_free(locale_filename);
+	g_free(unescaped_filename);
 	return ret;
 }
 
