@@ -1,8 +1,8 @@
 /*
  *      templates.h - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2005-2008 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
- *      Copyright 2006-2008 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
+ *      Copyright 2005-2012 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2006-2012 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -17,13 +17,21 @@
  *      You should have received a copy of the GNU General Public License
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
- *
- * $Id: templates.h 2775 2008-07-13 11:51:00Z eht16 $
  */
 
+/**
+ * @file templates.h
+ * Templates (prefs).
+ **/
+
+
 #ifndef GEANY_TEMPLATES_H
 #define GEANY_TEMPLATES_H 1
 
+G_BEGIN_DECLS
+
+#define GEANY_TEMPLATES_INDENT 3
+
 enum
 {
 	GEANY_TEMPLATE_GPL = 0,
@@ -35,13 +43,14 @@ enum
 };
 
 
+/** Template preferences. */
 typedef struct GeanyTemplatePrefs
 {
-	gchar			*developer;
-	gchar			*company;
-	gchar			*mail;
-	gchar			*initials;
-	gchar			*version;
+	gchar			*developer;	/**< Name */
+	gchar			*company;	/**< Company */
+	gchar			*mail;		/**< Email */
+	gchar			*initials;	/**< Initials */
+	gchar			*version;	/**< Initial version */
 	gchar			*year_format;
 	gchar			*date_format;
 	gchar			*datetime_format;
@@ -57,18 +66,20 @@ void templates_init(void);
 
 gchar *templates_get_template_fileheader(gint filetype_idx, const gchar *fname);
 
-gchar *templates_get_template_new_file(GeanyFiletype *ft);
+gchar *templates_get_template_changelog(GeanyDocument *doc);
 
-gchar *templates_get_template_changelog(void);
+gchar *templates_get_template_function(GeanyDocument *doc, const gchar *func_name);
 
-gchar *templates_get_template_generic(gint template);
+gchar *templates_get_template_licence(GeanyDocument *doc, gint licence_type);
 
-gchar *templates_get_template_function(gint filetype_idx, const gchar *func_name);
+void templates_replace_common(GString *tmpl, const gchar *fname,
+	GeanyFiletype *ft, const gchar *func_name);
 
-gchar *templates_get_template_licence(gint filetype_idx, gint licence_type);
-
-gchar *templates_replace_all(gchar *source, const gchar *year, const gchar *date);
+void templates_replace_valist(GString *text,
+	const gchar *first_wildcard, ...) G_GNUC_NULL_TERMINATED;
 
 void templates_free_templates(void);
 
+G_END_DECLS
+
 #endif
